@@ -435,6 +435,7 @@ class MaterialBuilder():
 class MaterialComponents():
   def FormatCustomNumber(tp, *args, **kwargs ):
     paper = TeleportElement(MaterialContent(elementType="Paper")) 
+    paper.content.attrs['elevation'] = kwargs.get("elevation", 0)
     paper.content.style = {"width" : "100%"}
     FComponent = TeleportComponent("FormatCustomNumber", paper)
     FComponent.addPropVariable("formatter", {"type":"func", 'defaultValue' : '''(props)=>{
@@ -443,10 +444,10 @@ class MaterialComponents():
                 'style':props.style,
                 'className':value.className,
                 'onValueChange': (v)=> {
-                    value.onChange( {'target':{'value':v.value } } );
+                    value.onChange( {'target':{'value':Number.isInteger(props.decimalscale) ? parseInt(v.value) : parseFloat(v.value)} } );
                 },
                 'onBlur' : (v)=> {
-                    props.onBlur( {'target':{'value':v.target.value } } );
+                    props.onBlur( {'target':{'value':Number.isInteger(props.decimalscale) ? parseInt(v.target.value) : parseFloat(v.target.value) } } );
                 },
                 'value' : Number.isInteger(props.decimalscale) ? parseInt(value.value) : parseFloat(value.value),
                 'isAllowed': (v)=> {
@@ -494,6 +495,7 @@ class MaterialComponents():
   def IconList(tp, *args, **kwargs ):
    
     Paper = TeleportElement(MaterialContent(elementType="Paper")) 
+    Paper.content.attrs['elevation'] = kwargs.get("elevation", 0)    
     Paper.content.style = {"width" : "100%"}        
     IComponent = TeleportComponent("IconListMaterial", Paper)        
     IComponent.addStateVariable("selected", {"type":"string", "defaultValue": {
@@ -688,7 +690,7 @@ class MaterialComponents():
         {
           "type": "stateChange",
           "modifies": "open_expansion",
-          "newState": "$self.props.always_open && false"
+          "newState": "$self.props.always_open"
         },
         {
             "type": "propCall2",
@@ -726,15 +728,17 @@ class MaterialComponents():
 
     Paper = TeleportElement(MaterialContent(elementType="Paper")) 
     Paper.addContent(ExpansionPanel)    
-    Paper.content.style = {"width" : "100%"}        
+    Paper.content.style = {"width" : "100%"} 
+    Paper.content.attrs['elevation'] = kwargs.get("elevation", 0)
+
     BLMComponent = TeleportComponent("ButtonListMaterial", Paper)        
 
     BLMComponent.addPropVariable("always_open", {"type":"bool", "defaultValue": False})
     BLMComponent.addPropVariable("hide_header", {"type":"bool", "defaultValue": False})
     BLMComponent.addPropVariable("options", {"type":"array", "defaultValue": []})
     BLMComponent.addPropVariable("onChange", {"type":"func", "defaultValue": "(e)=>{}"})
+    BLMComponent.addPropVariable("onDefaultValue", {"type":"func", "defaultValue": "(l,v)=>{let j=l.find((e)=>{return e.key==v}); if (j){ return j;} else { return {key:'', value:''};} }"})
     BLMComponent.addPropVariable("default_value", {"type":"string", "defaultValue": ""})
-    BLMComponent.addPropVariable("onDefaultValue", {"type":"func", "defaultValue": "(l,v)=>{return l.find((e)=>{return e.key==v})}"})
 
     BLMComponent.addStateVariable("open_expansion", {"type":"bool", "defaultValue": True})
     BLMComponent.addStateVariable("class_names", {"type":"object", "defaultValue": {}})
@@ -751,7 +755,8 @@ class MaterialComponents():
 
   def IntSwitch(tp, *args, **kwargs ):    
     Paper = TeleportElement(MaterialContent(elementType="Paper")) 
-    Paper.content.style = {"width" : "100%"}        
+    Paper.content.style = {"width" : "100%"}  
+    Paper.content.attrs['elevation'] = kwargs.get("elevation", 0)      
     IComponent = TeleportComponent("IntSwitch", Paper)        
 
     FormControl = TeleportElement(MaterialContent(elementType="FormControl"))
@@ -892,6 +897,7 @@ class MaterialComponents():
   def ColorSliders(tp, *args, **kwargs):
    
     Paper = TeleportElement(MaterialContent(elementType="Paper")) 
+    Paper.content.attrs['elevation'] = kwargs.get("elevation", 0)    
     Paper.content.style = {"width" : "100%"}        
     CComponent = TeleportComponent("ColorSliders", Paper)        
     CComponent.addStateVariable("colors", {"type":"array", "defaultValue": {
@@ -904,16 +910,19 @@ class MaterialComponents():
     eol = "\n"
     updateState = ""
     updateState += "(c,i,a,v)=>{ " + eol
-    updateState += "  c.setState({'colors' : " + eol
-    updateState += "    c.state.colors.map((e, ii)=>{ " + eol
-    updateState += "      if(ii==i) e[a]=v; " + eol
-    updateState += "      return e;" + eol
-    updateState += "    })" + eol
-    updateState += "  });" + eol
+    updateState += "  let new_colors = c.state.colors.map((e, ii)=>{" + eol
+    updateState += "    if(ii==i){" + eol
+    updateState += "      let clone = JSON.parse(JSON.stringify(e));" + eol
+    updateState += "      clone[a]=v;" + eol
+    updateState += "      return clone;" + eol
+    updateState += "    }" + eol
+    updateState += "    return e;" + eol
+    updateState += "  })" + eol
+    updateState += "  c.setState({'colors' : new_colors});" + eol
     updateState += "  if (c.timeout){" + eol
     updateState += "    clearTimeout(c.timeout)" + eol
     updateState += "  }" + eol
-    updateState += "  c.timeout = setTimeout(()=>{ c.props.onChange( {'target':{'value':c.props.colors}} ) }, 20);" + eol
+    updateState += "  c.timeout = setTimeout(()=>{ c.props.onChange( {'target':{'value':new_colors}} ) }, 20);" + eol
     updateState += "}" + eol
     CComponent.addPropVariable("updateState", {
         "type":"func", 
