@@ -4,22 +4,53 @@ import jsonschema
 import uuid, weakref, os
 from IPython.display import HTML, Javascript, display
 
+class TeleportCustomCode():
+    def __init__(self, *args, **kwargs):
+        self.ids = {}
+        self.code = []
+        
+    def buildReact(self, *args , **kwargs):
+      return self.__json__()
+      
+    def addCustomCode(self, id, asset):
+      if id in self.ids:
+        pass;
+      else:
+        self.ids[id] = len(self.ids)
+        self.code.append(asset)
+        
+    def __json__(self):
+      js = ""
+      for code in self.code:
+          js += code + "\n";
+      return js
+        
 class TeleportGlobals():
     def __init__(self, *args, **kwargs):
         self.settings = {'language' : 'en', 'title' : ''}
+        self.customCode = {
+            'head' : TeleportCustomCode(), 
+            'body' : TeleportCustomCode()
+        }
         self.assets = []
         self.meta = []
         self.manifest = {}
         self.ids = {}
         
     def __json__(self):
-        return {
+        jsn = {
             "settings" : self.settings,
+            "customCode" : {},
             "assets" : self.assets,
             "meta" : self.meta,
             "manifest" : self.manifest
         }
-    
+        if len (self.customCode['head'].code) > 0: 
+            jsn['customCode']['head'] = self.customCode['head'].__json__()
+        if len (self.customCode['body'].code) > 0 :
+            jsn['customCode']['body'] = self.customCode['body'].__json__()
+        return jsn
+        
     def __str__(self):
         return json.dumps(self.__json__())
         
@@ -40,7 +71,11 @@ class TeleportGlobals():
       else:
         self.ids[id] = len(self.ids)
         self.assets.append(asset)
-      
+        
+    def addCustomCode(self, id, code, position="body"):
+      if position in ["body", "head"]:
+        self.customCode[position].addCustomCode(id, code)
+  
         
 class TeleportNode():
   def __init__(self, *args, **kwargs):
@@ -85,6 +120,8 @@ class TeleportNode():
         react += self.content.buildReact( nativeRef=kwargs.get("nativeRef","") )
     return react
         
+  def getNodeTypes(self):
+    return set()
 
 class TeleportElement(TeleportNode):
   def __init__(self, content, *args, **kwargs):
@@ -95,6 +132,9 @@ class TeleportElement(TeleportNode):
   def addContent(self, child):
     self.content.children.append(child)
     
+  def getNodeTypes(self):
+    return self.content.getNodeTypes()
+    
 class TeleportConditional(TeleportNode):
   def __init__(self, content, *args, **kwargs):
     TeleportNode.__init__(self)
@@ -256,6 +296,9 @@ class TeleportComponent():
   def __str__(self):
       return json.dumps(self.__json__())
 
+  def getNodeTypes(self):
+    return self.node.content.getNodeTypes()
+
   def addNode(self, child):
     if (isinstance(child, TeleportNode) ):
       self.node.addContent(child)
@@ -349,7 +392,8 @@ class TeleportProject():
   def __init__(self, name, *args, **kwargs):
       self.project_name = name
       self.globals = TeleportGlobals();
-      self.root = TeleportComponent("MainComponent", TeleportElement(TeleportContent(elementType="container")));
+      content = kwargs.get("content", TeleportElement(TeleportContent(elementType="container")))
+      self.root = TeleportComponent("MainComponent", content);
       self.components = {};
       self.ref = uuid.uuid4()
 
@@ -391,7 +435,10 @@ class TeleportProject():
     react += "<title>" + self.project_name + "</title>\n"
     react += "<script src='https://cdn.plot.ly/plotly-latest.min.js'></script>\n"    
     react += "<script src='https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.6/require.min.js'></script>\n"
-    react += "<link rel='stylesheet' href='https://fonts.googleapis.com/icon?family=Material+Icons'/>\n"
+    react += "<link rel='stylesheet' href='https://fonts.googleapis.com/icon?family=Material+Icons'/>\n" 
+    react += "<script type='text/javascript'>\n"
+    react += self.globals.customCode['head'].buildReact() + "\n"
+    react += "</script>\n"
     react += "</head>\n"
     react += "  <body style='padding:0;margin:0;height:100%'>\n"
     react += "    <div id='root' style='height:100%'></div>\n"
@@ -418,6 +465,7 @@ class TeleportProject():
     react += "    requirejs(['plotlycomponent', 'plotly', 'math', 'number-format'], function(PlotlyComponent, Plotly, math, Format) {\n"
     react += "      window.React = React\n"
     react += "      const Plot = PlotlyComponent.default(Plotly);\n";
+    react += self.globals.customCode['body'].buildReact()
     react += self.globals.buildReact();
     react += self.root.buildReact(self.root.name_component)
     for k, v in self.components.items():
@@ -435,8 +483,9 @@ class TeleportProject():
     f = open(filename, "w")
     f.write(react)
     f.close()
-    return react
+    return react    
 
+    
   def displayWidget(self, *args, **kwargs):
     filename = "__TMPReactBuld.dat"
     self.buildReact(filename)
@@ -475,6 +524,16 @@ class TeleportContent():
     self.ref = uuid.uuid4()
     self.name =  kwargs.get("name", None)
     
+    
+  def getNodeTypes(self):
+    types = set()
+    if self.elementType != None:
+        types.add(self.elementType)
+    for c in self.children:
+        for v in c.getNodeTypes(): 
+            types.add(v) 
+    return types
+
   def __json__(self):
     tjson = {}
     if self.name != None:
@@ -793,10 +852,12 @@ class NanohubUtils():
     js += "}" + eol;
 
         
-    tp.globals.addAsset(method_name, {
-      "type": "script",
-      "content": js
-    })    
+    #tp.globals.addAsset(method_name, {
+    #  "type": "script",
+    #  "content": js
+    #}) 
+    tp.globals.addCustomCode(method_name, js)    
+    
 
     js = ""
     js += " LocalForage."+method_name+" = '"+driver_name+"'; " + eol
@@ -807,10 +868,11 @@ class NanohubUtils():
     js += " }).then(function() {" + eol
     js += " });" + eol
     
-    tp.globals.addAsset("_" + method_name, {
-      "type": "script",
-      "content": js
-    })
+    #tp.globals.addAsset("_" + method_name, {
+    #  "type": "script",
+    #  "content": js
+    #})
+    tp.globals.addCustomCode("_" + method_name, js)    
 
     return [
       {
@@ -916,15 +978,18 @@ class NanohubUtils():
     js += "  };" + eol;
     js += "};" + eol;
 
-    tp.globals.addAsset(method_name, {
-      "type": "script",
-      "content": js
-    })    
+    #tp.globals.addAsset(method_name, {
+    #  "type": "script",
+    #  "content": js
+    #})    
+    tp.globals.addCustomCode(method_name, js)    
 
-    tp.globals.addAsset(store_name, {
-      "type": "script",
-      "content": "const " + store_name + " = " + method_name + "(() => " + storage_name + ");" + eol
-    })
+    #tp.globals.addAsset(store_name, {
+    #  "type": "script",
+    #  "content": "const " + store_name + " = " + method_name + "(() => " + storage_name + ");" + eol
+    #})
+    
+    tp.globals.addCustomCode(store_name, "const " + store_name + " = " + method_name + "(() => " + storage_name + ");" + eol)    
 
     return [
       {
